'use client';

import * as React from 'react';
import PropTypes from 'prop-types';
import { useComponentRenderer } from '../../utils/useComponentRenderer';
import { mergeReactProps } from '../../utils/mergeReactProps';
import { FormRootContext } from './FormRootContext';
import { useEventCallback } from '../../utils/useEventCallback';

/**
 *
 * Demos:
 *
 * - [Form](https://base-ui.netlify.app/components/react-form/)
 *
 * API:
 *
 * - [FormRoot API](https://base-ui.netlify.app/components/react-form/#api-reference-FormRoot)
 */
import { jsx as _jsx } from "react/jsx-runtime";
const FormRoot = /*#__PURE__*/React.forwardRef(function FormRoot(props, forwardedRef) {
  const {
    render,
    className,
    errors,
    onClearErrors: onClearErrorsProp,
    onSubmit: onSubmitProp,
    ...otherProps
  } = props;
  const formRef = React.useRef({
    fields: new Map()
  });
  const onSubmit = useEventCallback(onSubmitProp || (() => {}));
  const onClearErrors = useEventCallback(onClearErrorsProp || (() => {}));
  const getFormProps = React.useCallback((externalProps = {}) => mergeReactProps(externalProps, {
    noValidate: true,
    onSubmit(event) {
      let values = Array.from(formRef.current.fields.values());

      // Async validation isn't supported to stop the submit event.
      values.forEach(field => {
        field.validate();
      });
      values = Array.from(formRef.current.fields.values());
      const invalidFields = values.filter(field => !field.validityData.state.valid);
      if (invalidFields.length) {
        event.preventDefault();
        invalidFields[0]?.controlRef.current?.focus();
      } else {
        onSubmit(event);
      }
    }
  }), [onSubmit]);
  React.useEffect(() => {
    const invalidFields = Array.from(formRef.current.fields.values()).filter(field => field.validityData.state.valid === false);
    if (invalidFields.length) {
      invalidFields[0]?.controlRef.current?.focus();
    }
  }, [errors]);
  const ownerState = React.useMemo(() => ({}), []);
  const {
    renderElement
  } = useComponentRenderer({
    propGetter: getFormProps,
    render: render ?? 'form',
    ref: forwardedRef,
    ownerState,
    className,
    extraProps: otherProps
  });
  const contextValue = React.useMemo(() => ({
    formRef,
    errors: errors ?? {},
    onClearErrors
  }), [formRef, errors, onClearErrors]);
  return /*#__PURE__*/_jsx(FormRootContext.Provider, {
    value: contextValue,
    children: renderElement()
  });
});
process.env.NODE_ENV !== "production" ? FormRoot.propTypes /* remove-proptypes */ = {
  // ┌────────────────────────────── Warning ──────────────────────────────┐
  // │ These PropTypes are generated from the TypeScript type definitions. │
  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │
  // └─────────────────────────────────────────────────────────────────────┘
  /**
   * @ignore
   */
  children: PropTypes.node,
  /**
   * Class names applied to the element or a function that returns them based on the component's state.
   */
  className: PropTypes.oneOfType([PropTypes.func, PropTypes.string]),
  /**
   * Object of error messages with each key mapping to the `name` prop of a Field control, usually
   * from server-side validation.
   */
  errors: PropTypes.object,
  /**
   * Callback fired when the external server-side `error` messages should be cleared.
   */
  onClearErrors: PropTypes.func,
  /**
   * @ignore
   */
  onSubmit: PropTypes.func,
  /**
   * A function to customize rendering of the component.
   */
  render: PropTypes.oneOfType([PropTypes.element, PropTypes.func])
} : void 0;
export { FormRoot };