'use client';

import * as React from 'react';
import PropTypes from 'prop-types';
import { useMenuArrow } from './useMenuArrow';
import { useMenuPositionerContext } from '../Positioner/MenuPositionerContext';
import { useMenuRootContext } from '../Root/MenuRootContext';
import { commonStyleHooks } from '../utils/commonStyleHooks';
import { useComponentRenderer } from '../../utils/useComponentRenderer';
import { useForkRef } from '../../utils/useForkRef';
/**
 * Renders an arrow that points to the center of the anchor element.
 *
 * Demos:
 *
 * - [Menu](https://base-ui.netlify.app/components/react-menu/)
 *
 * API:
 *
 * - [MenuArrow API](https://base-ui.netlify.app/components/react-menu/#api-reference-MenuArrow)
 */
const MenuArrow = /*#__PURE__*/React.forwardRef(function MenuArrow(props, forwardedRef) {
  const {
    className,
    render,
    hideWhenUncentered = false,
    ...otherProps
  } = props;
  const {
    open
  } = useMenuRootContext();
  const {
    arrowRef,
    side,
    alignment,
    arrowUncentered,
    arrowStyles
  } = useMenuPositionerContext();
  const {
    getArrowProps
  } = useMenuArrow({
    arrowStyles,
    hidden: hideWhenUncentered && arrowUncentered
  });
  const ownerState = React.useMemo(() => ({
    open,
    side,
    alignment,
    arrowUncentered
  }), [open, side, alignment, arrowUncentered]);
  const mergedRef = useForkRef(arrowRef, forwardedRef);
  const {
    renderElement
  } = useComponentRenderer({
    propGetter: getArrowProps,
    render: render ?? 'div',
    className,
    ownerState,
    ref: mergedRef,
    extraProps: otherProps,
    customStyleHookMapping: commonStyleHooks
  });
  return renderElement();
});
process.env.NODE_ENV !== "production" ? MenuArrow.propTypes /* remove-proptypes */ = {
  // ┌────────────────────────────── Warning ──────────────────────────────┐
  // │ These PropTypes are generated from the TypeScript type definitions. │
  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │
  // └─────────────────────────────────────────────────────────────────────┘
  /**
   * @ignore
   */
  children: PropTypes.node,
  /**
   * Class names applied to the element or a function that returns them based on the component's state.
   */
  className: PropTypes.oneOfType([PropTypes.func, PropTypes.string]),
  /**
   * If `true`, the arrow is hidden when it can't point to the center of the anchor element.
   * @default false
   */
  hideWhenUncentered: PropTypes.bool,
  /**
   * A function to customize rendering of the component.
   */
  render: PropTypes.oneOfType([PropTypes.element, PropTypes.func])
} : void 0;
export { MenuArrow };