'use client';

import * as React from 'react';
import PropTypes from 'prop-types';
import { useFloatingTree, useListItem } from '@floating-ui/react';
import { useMenuRadioItem } from './useMenuRadioItem';
import { useMenuRootContext } from '../Root/MenuRootContext';
import { useComponentRenderer } from '../../utils/useComponentRenderer';
import { useId } from '../../utils/useId';
import { useForkRef } from '../../utils/useForkRef';
import { useMenuRadioGroupContext } from '../RadioGroup/MenuRadioGroupContext';
import { MenuRadioItemContext } from './MenuRadioItemContext';
import { jsx as _jsx } from "react/jsx-runtime";
const customStyleHookMapping = {
  checked: value => ({
    'data-radioitem': value ? 'checked' : 'unchecked'
  })
};
const InnerMenuRadioItem = /*#__PURE__*/React.memo(/*#__PURE__*/React.forwardRef(function InnerMenuItem(props, forwardedRef) {
  const {
    checked,
    setChecked,
    className,
    closeOnClick = false,
    disabled = false,
    highlighted,
    id,
    menuEvents,
    propGetter,
    render,
    treatMouseupAsClick,
    typingRef,
    ...other
  } = props;
  const {
    getRootProps
  } = useMenuRadioItem({
    checked,
    setChecked,
    closeOnClick,
    disabled,
    highlighted,
    id,
    menuEvents,
    ref: forwardedRef,
    treatMouseupAsClick,
    typingRef
  });
  const ownerState = {
    disabled,
    highlighted,
    checked
  };
  const {
    renderElement
  } = useComponentRenderer({
    render: render || 'div',
    className,
    ownerState,
    propGetter: externalProps => propGetter(getRootProps(externalProps)),
    customStyleHookMapping,
    extraProps: other
  });
  return renderElement();
}));

/**
 * An unstyled menu item to be used within a Menu.
 *
 * Demos:
 *
 * - [Menu](https://base-ui.netlify.app/components/react-menu/)
 *
 * API:
 *
 * - [MenuRadioItem API](https://base-ui.netlify.app/components/react-menu/#api-reference-MenuRadioItem)
 */
const MenuRadioItem = /*#__PURE__*/React.forwardRef(function MenuRadioItem(props, forwardedRef) {
  const {
    id: idProp,
    value,
    label,
    disabled = false,
    ...other
  } = props;
  const itemRef = React.useRef(null);
  const listItem = useListItem({
    label: label ?? itemRef.current?.innerText
  });
  const mergedRef = useForkRef(forwardedRef, listItem.ref, itemRef);
  const {
    getItemProps,
    activeIndex,
    clickAndDragEnabled,
    typingRef
  } = useMenuRootContext();
  const id = useId(idProp);
  const highlighted = listItem.index === activeIndex;
  const {
    events: menuEvents
  } = useFloatingTree();
  const {
    value: selectedValue,
    setValue: setSelectedValue
  } = useMenuRadioGroupContext();

  // This wrapper component is used as a performance optimization.
  // MenuRadioItem reads the context and re-renders the actual MenuRadioItem
  // only when it needs to.

  const checked = selectedValue === value;
  const setChecked = React.useCallback(event => {
    setSelectedValue(value, event);
  }, [setSelectedValue, value]);
  const contextValue = React.useMemo(() => ({
    checked,
    highlighted,
    disabled
  }), [checked, highlighted, disabled]);
  return /*#__PURE__*/_jsx(MenuRadioItemContext.Provider, {
    value: contextValue,
    children: /*#__PURE__*/_jsx(InnerMenuRadioItem, {
      ...other,
      id: id,
      ref: mergedRef,
      highlighted: highlighted,
      menuEvents: menuEvents,
      propGetter: getItemProps,
      treatMouseupAsClick: clickAndDragEnabled,
      checked: selectedValue === value,
      setChecked: setChecked,
      typingRef: typingRef
    })
  });
});
process.env.NODE_ENV !== "production" ? MenuRadioItem.propTypes /* remove-proptypes */ = {
  // ┌────────────────────────────── Warning ──────────────────────────────┐
  // │ These PropTypes are generated from the TypeScript type definitions. │
  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │
  // └─────────────────────────────────────────────────────────────────────┘
  /**
   * @ignore
   */
  children: PropTypes.node,
  /**
   * If `true`, the menu will close when the menu item is clicked.
   *
   * @default true
   */
  closeOnClick: PropTypes.bool,
  /**
   * If `true`, the menu item will be disabled.
   * @default false
   */
  disabled: PropTypes.bool,
  /**
   * The id of the menu item.
   */
  id: PropTypes.string,
  /**
   * A text representation of the menu item's content.
   * Used for keyboard text navigation matching.
   */
  label: PropTypes.string,
  /**
   * The click handler for the menu item.
   */
  onClick: PropTypes.func,
  /**
   * Value of the radio item.
   * This is the value that will be set in the MenuRadioGroup when the item is selected.
   */
  value: PropTypes.any.isRequired
} : void 0;
export { MenuRadioItem };