'use client';

import * as React from 'react';
import PropTypes from 'prop-types';
import { useFloatingTree, useListItem } from '@floating-ui/react';
import { useMenuRootContext } from '../Root/MenuRootContext';
import { commonStyleHooks } from '../utils/commonStyleHooks';
import { useId } from '../../utils/useId';
import { useComponentRenderer } from '../../utils/useComponentRenderer';
import { useSubmenuTrigger } from './useSubmenuTrigger';
import { useForkRef } from '../../utils/useForkRef';

/**
 *
 * Demos:
 *
 * - [Menu](https://base-ui.netlify.app/components/react-menu/)
 *
 * API:
 *
 * - [SubmenuTrigger API](https://base-ui.netlify.app/components/react-menu/#api-reference-SubmenuTrigger)
 */
const SubmenuTrigger = /*#__PURE__*/React.forwardRef(function SubmenuTriggerComponent(props, forwardedRef) {
  const {
    render,
    className,
    disabled = false,
    label,
    id: idProp,
    ...other
  } = props;
  const id = useId(idProp);
  const {
    getTriggerProps,
    parentContext,
    setTriggerElement,
    clickAndDragEnabled,
    open,
    typingRef
  } = useMenuRootContext();
  if (parentContext === null) {
    throw new Error('Base UI: ItemTrigger must be placed in a nested Menu.');
  }
  const {
    activeIndex,
    getItemProps
  } = parentContext;
  const item = useListItem();
  const highlighted = activeIndex === item.index;
  const mergedRef = useForkRef(forwardedRef, item.ref);
  const {
    events: menuEvents
  } = useFloatingTree();
  const {
    getRootProps
  } = useSubmenuTrigger({
    id,
    highlighted,
    ref: mergedRef,
    disabled,
    menuEvents,
    setTriggerElement,
    treatMouseupAsClick: clickAndDragEnabled,
    typingRef
  });
  const ownerState = {
    disabled,
    highlighted,
    open
  };
  const {
    renderElement
  } = useComponentRenderer({
    render: render || 'div',
    className,
    ownerState,
    propGetter: externalProps => getTriggerProps(getItemProps(getRootProps(externalProps))),
    customStyleHookMapping: commonStyleHooks,
    extraProps: other
  });
  return renderElement();
});
process.env.NODE_ENV !== "production" ? SubmenuTrigger.propTypes /* remove-proptypes */ = {
  // ┌────────────────────────────── Warning ──────────────────────────────┐
  // │ These PropTypes are generated from the TypeScript type definitions. │
  // │ To update them, edit the TypeScript types and run `pnpm proptypes`. │
  // └─────────────────────────────────────────────────────────────────────┘
  /**
   * @ignore
   */
  children: PropTypes.node,
  /**
   * Class names applied to the element or a function that returns them based on the component's state.
   */
  className: PropTypes.oneOfType([PropTypes.func, PropTypes.string]),
  /**
   * If `true`, the menu item will be disabled.
   * @default false
   */
  disabled: PropTypes.bool,
  /**
   * @ignore
   */
  id: PropTypes.string,
  /**
   * A text representation of the menu item's content.
   * Used for keyboard text navigation matching.
   */
  label: PropTypes.string,
  /**
   * @ignore
   */
  onClick: PropTypes.func,
  /**
   * A function to customize rendering of the component.
   */
  render: PropTypes.oneOfType([PropTypes.element, PropTypes.func])
} : void 0;
export { SubmenuTrigger };