"use strict";
'use client';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useAnchorPositioning = useAnchorPositioning;
var React = _interopRequireWildcard(require("react"));
var _react2 = require("@floating-ui/react");
var _utils = require("@floating-ui/utils");
var _useEnhancedEffect = require("./useEnhancedEffect");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
/**
 * Provides standardized anchor positioning behavior for floating elements. Wraps Floating UI's
 * `useFloating` hook.
 * @ignore - internal hook.
 */
function useAnchorPositioning(params = {}) {
  const {
    anchor,
    floatingRootContext,
    positionMethod = 'absolute',
    side = 'top',
    sideOffset = 0,
    alignment = 'center',
    alignmentOffset = 0,
    collisionBoundary,
    collisionPadding = 5,
    hideWhenDetached = false,
    fallbackAxisSideDirection = 'start',
    sticky = false,
    keepMounted = false,
    arrowPadding = 5,
    mounted = true,
    trackAnchor = true,
    nodeId
  } = params;
  const placement = alignment === 'center' ? side : `${side}-${alignment}`;
  const commonCollisionProps = {
    boundary: collisionBoundary,
    padding: collisionPadding
  };

  // Using a ref assumes that the arrow element is always present in the DOM for the lifetime of the
  // tooltip. If this assumption ends up being false, we can switch to state to manage the arrow's
  // presence.
  const arrowRef = React.useRef(null);
  const middleware = [(0, _react2.offset)({
    mainAxis: sideOffset,
    crossAxis: alignmentOffset,
    alignmentAxis: alignmentOffset
  })];
  const flipMiddleware = (0, _react2.flip)({
    ...commonCollisionProps,
    fallbackAxisSideDirection
  });
  const shiftMiddleware = (0, _react2.shift)({
    ...commonCollisionProps,
    crossAxis: sticky,
    limiter: sticky ? undefined : (0, _react2.limitShift)(() => {
      if (!arrowRef.current) {
        return {};
      }
      const {
        height
      } = arrowRef.current.getBoundingClientRect();
      return {
        offset: height / 2 + (typeof collisionPadding === 'number' ? collisionPadding : 0)
      };
    })
  });

  // https://floating-ui.com/docs/flip#combining-with-shift
  if (alignment !== 'center') {
    middleware.push(flipMiddleware, shiftMiddleware);
  } else {
    middleware.push(shiftMiddleware, flipMiddleware);
  }
  middleware.push((0, _react2.size)({
    ...commonCollisionProps,
    apply({
      elements: {
        floating
      },
      rects: {
        reference
      },
      availableWidth,
      availableHeight
    }) {
      Object.entries({
        '--available-width': `${availableWidth}px`,
        '--available-height': `${availableHeight}px`,
        '--anchor-width': `${reference.width}px`,
        '--anchor-height': `${reference.height}px`
      }).forEach(([key, value]) => {
        floating.style.setProperty(key, value);
      });
    }
  }), (0, _react2.arrow)(() => ({
    // `transform-origin` calculations rely on an element existing. If the arrow hasn't been set,
    // we'll create a fake element.
    element: arrowRef.current || document.createElement('div'),
    padding: arrowPadding
  }), [arrowPadding]), hideWhenDetached && (0, _react2.hide)(), {
    name: 'transformOrigin',
    fn({
      elements,
      middlewareData,
      placement: renderedPlacement
    }) {
      const currentRenderedSide = (0, _utils.getSide)(renderedPlacement);
      const arrowEl = arrowRef.current;
      const arrowX = middlewareData.arrow?.x ?? 0;
      const arrowY = middlewareData.arrow?.y ?? 0;
      const arrowWidth = arrowEl?.clientWidth ?? sideOffset;
      const arrowHeight = arrowEl?.clientHeight ?? sideOffset;
      const transformX = arrowX + arrowWidth / 2;
      const transformY = arrowY + arrowHeight;
      const transformOrigin = {
        top: `${transformX}px calc(100% + ${arrowHeight}px)`,
        bottom: `${transformX}px ${-arrowHeight}px`,
        left: `calc(100% + ${arrowHeight}px) ${transformY}px`,
        right: `${-arrowHeight}px ${transformY}px`
      }[currentRenderedSide];
      elements.floating.style.setProperty('--transform-origin', transformOrigin);
      return {};
    }
  });
  const {
    refs,
    elements,
    floatingStyles,
    middlewareData,
    update,
    placement: renderedPlacement,
    context: positionerContext
  } = (0, _react2.useFloating)({
    rootContext: floatingRootContext,
    placement,
    middleware,
    strategy: positionMethod,
    whileElementsMounted: keepMounted || !trackAnchor ? undefined : _react2.autoUpdate,
    nodeId
  });
  const registeredPositionReferenceRef = React.useRef(null);
  (0, _useEnhancedEffect.useEnhancedEffect)(() => {
    const resolvedAnchor = typeof anchor === 'function' ? anchor() : anchor;
    if (resolvedAnchor) {
      const unwrappedElement = isRef(resolvedAnchor) ? resolvedAnchor.current : resolvedAnchor;
      refs.setPositionReference(unwrappedElement);
      registeredPositionReferenceRef.current = unwrappedElement;
    }
  }, [refs, anchor]);
  React.useEffect(() => {
    // Refs from parent components are set after useLayoutEffect runs and are available in useEffect.
    // Therefore, if the anchor is a ref, we need to update the position reference in useEffect.
    if (typeof anchor === 'function') {
      return;
    }
    if (isRef(anchor) && anchor.current !== registeredPositionReferenceRef.current) {
      refs.setPositionReference(anchor.current);
      registeredPositionReferenceRef.current = anchor.current;
    }
  }, [refs, anchor]);
  React.useEffect(() => {
    if (keepMounted && mounted && elements.domReference && elements.floating) {
      return (0, _react2.autoUpdate)(elements.domReference, elements.floating, update);
    }
    return undefined;
  }, [keepMounted, mounted, elements, update]);
  const renderedSide = (0, _utils.getSide)(renderedPlacement);
  const renderedAlignment = (0, _utils.getAlignment)(renderedPlacement) || 'center';
  const hidden = Boolean(hideWhenDetached && middlewareData.hide?.referenceHidden);
  const positionerStyles = React.useMemo(() => ({
    ...floatingStyles,
    ...(hidden && {
      visibility: 'hidden'
    })
  }), [floatingStyles, hidden]);
  const arrowStyles = React.useMemo(() => ({
    position: 'absolute',
    top: middlewareData.arrow?.y,
    left: middlewareData.arrow?.x
  }), [middlewareData.arrow]);
  const arrowUncentered = middlewareData.arrow?.centerOffset !== 0;
  return React.useMemo(() => ({
    positionerStyles,
    arrowStyles,
    arrowRef,
    arrowUncentered,
    renderedSide,
    renderedAlignment,
    hidden,
    refs,
    positionerContext
  }), [positionerStyles, arrowStyles, arrowRef, arrowUncentered, renderedSide, renderedAlignment, hidden, refs, positionerContext]);
}
function isRef(param) {
  return param != null && 'current' in param;
}